@@ -27,6 +27,87 @@ signatures. They can be run by calling the :func:`DXApplet.run` method.
 import dxpy
 from dxpy.bindings import *
 
+class DXExecutable:
+    ''' Methods in DXExecutable are used by both DXApp and DXApplet.
+    '''
+    def __init__(self, *args, **kwargs):
+        raise NotImplementedError("This class is a mix-in. Use DXApp or DXApplet instead.")
+
+    def run(self, executable_input, project=None, folder="/", name=None, instance_type=None,
+            depends_on=None, details=None, delay_workspace_destruction=None, **kwargs):
+        '''
+        :param executable_input: Hash of the executable's input arguments
+        :type executable_input: dict
+        :param project: Project ID of the project context
+        :type project: string
+        :param folder: Folder in which executable's outputs will be placed in *project*
+        :type folder: string
+        :param name: Name for the new job (default is "<name of the executable>")
+        :type name: string
+        :param instance_type: Instance type on which the jobs will be run, or a dict mapping function names to instance type requests
+        :type instance_type: string or dict
+        :param depends_on: List of data objects or jobs to wait that need to enter the "closed" or "done" states, respectively, before the new job will be run; each element in the list can either be a dxpy handler or a string ID
+        :type depends_on: list
+        :param details: Details to set for the job
+        :type details: dict or list
+        :param delay_workspace_destruction: Whether to keep the job's temporary workspace around for debugging purposes for 3 days after it succeeds or fails
+        :type delay_workspace_destruction: boolean
+        :returns: Object handler of the newly created job
+        :rtype: :class:`~dxpy.bindings.dxjob.DXJob`
+
+        Creates a new job that executes the function "main" of this executable with
+        the given input *executable_input*.
+
+        '''
+        if project is None:
+            project = dxpy.WORKSPACE_ID
+
+        run_input = {"input": executable_input,
+                     "folder": folder}
+        if name is not None:
+            run_input["name"] = name
+        if instance_type is not None:
+            if isinstance(instance_type, basestring):
+                run_input["systemRequirements"] = {"*": {"instanceType": instance_type}}
+            elif isinstance(instance_type, dict):
+                run_input["systemRequirements"] = {stage: {"instanceType": stage_inst} for stage, stage_inst in instance_type.iteritems()}
+            else:
+                raise DXError('Expected instance_type field to be either a string or a dict')
+
+        if depends_on is not None:
+            run_input["dependsOn"] = []
+            if isinstance(depends_on, list):
+                for item in depends_on:
+                    if isinstance(item, DXJob) or isinstance(item, DXDataObject):
+                        if item.get_id() is None:
+                            raise DXError('A dxpy handler given in depends_on does not have an ID set')
+                        run_input["dependsOn"].append(item.get_id())
+                    elif isinstance(item, basestring):
+                        run_input['dependsOn'].append(item)
+                    else:
+                        raise DXError('Expected elements of depends_on to only be either instances of DXJob or DXDataObject, or strings')
+            else:
+                raise DXError('Expected depends_on field to be a list')
+
+        if details is not None:
+            run_input["details"] = details
+
+        if delay_workspace_destruction is not None:
+            run_input["delayWorkspaceDestruction"] = delay_workspace_destruction
+
+        if dxpy.JOB_ID is None:
+            run_input["project"] = project
+
+        if isinstance(self, DXApplet):
+            return DXJob(dxpy.api.applet_run(self._dxid, run_input, **kwargs)["id"])
+        elif self._dxid is not None:
+            return DXJob(dxpy.api.app_run(self._dxid, input_params=run_input, **kwargs)["id"])
+        else:
+            return DXJob(dxpy.api.app_run('app-' + self._name, alias=self._alias,
+                                          input_params=run_input,
+                                          **kwargs)["id"])
+
+
 ############
 # DXApplet #
 ############
@@ -36,7 +117,7 @@ def _makeNonexistentAPIWrapper(method):
         raise DXError("Wrapper for " + method + " does not exist")
     return nonexistentAPIWrapper
 
-class DXApplet(DXDataObject):
+class DXApplet(DXDataObject, DXExecutable):
     '''
     Remote applet object handler.
 
@@ -118,70 +199,30 @@ class DXApplet(DXDataObject):
         """
         return dxpy.api.applet_get(self._dxid, **kwargs)
 
-    def run(self, applet_input, project=None, folder="/", name=None, instance_type=None,
-            depends_on=None, details=None, delay_workspace_destruction=None, **kwargs):
-        '''
-        :param applet_input: Hash of the applet's input arguments
-        :type applet_input: dict
-        :param project: Project ID of the project context
-        :type project: string
-        :param folder: Folder in which applet's outputs will be placed in *project*
-        :type folder: string
-        :param name: Name for the new job (default is "<name of the applet>")
-        :type name: string
-        :param instance_type: Instance type on which the job with entry point "main" will be run, or a dict mapping function names to instance type requests
-        :type instance_type: string or dict
-        :param depends_on: List of data objects or jobs to wait that need to enter the "closed" or "done" states, respectively, before the new job will be run; each element in the list can either be a dxpy handler or a string ID
-        :type depends_on: list
-        :param details: Details to set for the job
-        :type details: dict or list
-        :param delay_workspace_destruction: Whether to keep the job's temporary workspace around for debugging purposes for 3 days after it succeeds or fails
-        :type delay_workspace_destruction: boolean
-        :returns: Object handler of the newly created job
-        :rtype: :class:`~dxpy.bindings.dxjob.DXJob`
-
+    def run(self, applet_input, project=None, folder="/", name=None,
+            instance_type=None, depends_on=None, details=None,
+            delay_workspace_destruction=None, **kwargs):
+        """
         Creates a new job that executes the function "main" of this applet with
         the given input *applet_input*.
 
-        '''
-        if project is None:
-            project = dxpy.WORKSPACE_ID
-
-        run_input = {"input": applet_input,
-                     "folder": folder}
-        if name is not None:
-            run_input["name"] = name
-        if instance_type is not None:
-            if isinstance(instance_type, basestring):
-                run_input["systemRequirements"] = {"main": {"instanceType": instance_type}}
-            elif isinstance(instance_type, dict):
-                run_input["systemRequirements"] = {stage: {"instanceType": stage_inst} for stage, stage_inst in instance_type.iteritems()}
-            else:
-                raise DXError('Expected instance_type field to be either a string or a dict')
-
-        if depends_on is not None:
-            run_input["dependsOn"] = []
-            if isinstance(depends_on, list):
-                for item in depends_on:
-                    if isinstance(item, DXJob) or isinstance(item, DXDataObject):
-                        if item.get_id() is None:
-                            raise DXError('A dxpy handler given in depends_on does not have an ID set')
-                        run_input["dependsOn"].append(item.get_id())
-                    elif isinstance(item, basestring):
-                        run_input['dependsOn'].append(item)
-                    else:
-                        raise DXError('Expected elements of depends_on to only be either instances of DXJob or DXDataObject, or strings')
-            else:
-                raise DXError('Expected depends_on field to be a list')                    
-
-        if details is not None:
-            run_input["details"] = details
-
-        if delay_workspace_destruction is not None:
-            run_input["delayWorkspaceDestruction"] = delay_workspace_destruction
-
-        if dxpy.JOB_ID is None:
-            run_input["project"] = project
-
-        return DXJob(dxpy.api.applet_run(self._dxid, run_input,
-                                         **kwargs)["id"])
+        See :meth:`dxpy.bindings.dxapplet.DXExecutable.run` for the meanings of
+        the keyword args.
+        """
+        # Rename applet_input arg to executable_input to preserve API
+        # compatibility when calling DXApplet.run(applet_input=...)
+        #
+        # If we don't allow the possibility of calling the named args
+        # positionally, we can change this to:
+        #   return super(DXApplet, self).run(applet_input, **kwargs)
+        return super(DXApplet, self).run(
+            applet_input,
+            project=project,
+            folder=folder,
+            name=name,
+            instance_type=instance_type,
+            depends_on=depends_on,
+            details=details,
+            delay_workspace_destruction=delay_workspace_destruction,
+            **kwargs
+        )
