@@ -83,6 +83,29 @@ def overrideEnvironment(**kwargs):
             env[key] = kwargs[key]
     return env
 
+def makeGenomeObject():
+    # NOTE: for these tests we don't upload a full sequence file (which
+    # would be huge, for hg19). Importers and exporters that need to
+    # look at the full sequence file can't be run on this test
+    # contigset.
+    sequence_file = dxpy.upload_string("")
+
+    genome_record = dxpy.new_dxrecord()
+    genome_record.set_details({
+        "flat_sequence_file": {"$dnanexus_link": sequence_file.get_id()},
+        "contigs": {
+            "offsets": [0],
+            "names": ["chr1"],
+            "sizes": [249250621]
+        }
+    })
+    genome_record.add_types(["ContigSet"])
+    genome_record.close()
+
+    sequence_file.wait_on_close()
+
+    return genome_record.get_id()
+
 class TestDXClient(DXTestCase):
     def test_dx_actions(self):
         with self.assertRaises(subprocess.CalledProcessError):
@@ -143,7 +166,7 @@ class TestDXClient(DXTestCase):
         self.assertEqual(record_id, run(u"dx ls :foo2 --brief").strip())
         self.assertEqual({"hello": "world"}, json.loads(run(u"dx get -o - :foo2")))
 
-        second_record_id = run(u"dx new record :somenewfolder/foo --parents --brief")
+        second_record_id = run(u"dx new record :somenewfolder/foo --parents --brief").strip()
         self.assertEqual(second_record_id, run(u"dx ls :somenewfolder/foo --brief").strip())
 
         # describe
@@ -165,6 +188,11 @@ class TestDXClient(DXTestCase):
         run(u"dx find jobs --project :")
         run(u"dx find data --project :")
 
+    def test_dx_describe_project(self):
+        describe_output = run(u"dx describe :").strip()
+        self.assertTrue(re.search(r'ID\s+%s.*\n.*\nName\s+dxclient_test_pr\xc3\xb6ject' % (self.project,),
+                                  describe_output))
+
     def test_dx_remove_project_by_name(self):
         # TODO: this test makes no use of the DXTestCase-provided
         # project.
@@ -611,6 +639,31 @@ class TestDXBuildReportHtml(unittest.TestCase):
         run(u"dx rm {record} {file}".format(record=report["recordId"], file=fileId))
 
 
+class TestDXBedToSpans(DXTestCase):
+    def setUp(self):
+        super(TestDXBedToSpans, self).setUp()
+        self.bed = """chr1\t127471196\t127472363\tPos1\t0\t+\t127471196\t127472363\t255,0,0
+"""
+        self.expected_tsv = """chr:string\tlo:int32\thi:int32\tname:string\tscore:float\tstrand:string\tthick_start:int32\tthick_end:int32\titem_rgb:string\r
+chr1\t127471196\t127472363\tPos1\t0\t+\t127471196\t127472363\t255,0,0\r
+"""
+        self.tempdir = tempfile.mkdtemp()
+        self.genome_id = makeGenomeObject()
+    def tearDown(self):
+        shutil.rmtree(self.tempdir)
+        super(TestDXBedToSpans, self).tearDown()
+    def test_bed_to_spans_conversion(self):
+        tempfile1 = os.path.join(self.tempdir, 'test1.bed')
+        with open(tempfile1, 'w') as f:
+            f.write(self.bed)
+        output = json.loads(run('dx-bed-to-spans {f} {g}'.format(f=tempfile1, g=self.genome_id)).strip().split('\n')[-1])
+        table_id = output[0]['$dnanexus_link']
+        run('dx wait {g}'.format(g=table_id))
+        self.assertEquals(run('dx export tsv -o - {g}'.format(g=table_id)), self.expected_tsv)
+    #def test_bed_spans_roundtrip(self):
+    #    pass
+
+
 class TestDXFastQToReads(DXTestCase):
     def setUp(self):
         super(TestDXFastQToReads, self).setUp()
@@ -662,6 +715,110 @@ NGTAACTCCTCTTTGCAACACCACAGCCATCGCCCCCTACCTCCTTGCCAATCCCAGGCTCCTCTCCTGATGGTAACATT
         self.assertEquals(open(os.path.join(self.tempdir, 'roundtrip.fq')).read(), round_tripped_fastq)
 
 
+class TestDXGtfToGenes(DXTestCase):
+    def setUp(self):
+        super(TestDXGtfToGenes, self).setUp()
+        self.expected_gtf = """chr1\t.\texon\t101\t200\t.\t+\t.\tgene_id ""; transcript_id "mytranscript-noncoding"
+chr1\t.\tCDS\t151\t200\t.\t+\t0\tgene_id "mygene-coding"; transcript_id "mytranscript-coding"
+"""
+        self.tempdir = tempfile.mkdtemp()
+        self.genome_id = makeGenomeObject()
+    def tearDown(self):
+        shutil.rmtree(self.tempdir)
+        super(TestDXGtfToGenes, self).tearDown()
+    def test_genes_to_gtf_conversion(self):
+        genes_table = dxpy.new_dxgtable([
+            dxpy.DXGTable.make_column_desc("type", "string"),
+            dxpy.DXGTable.make_column_desc("span_id", "int64"),
+            dxpy.DXGTable.make_column_desc("name", "string"),
+            dxpy.DXGTable.make_column_desc("strand", "string"),
+            dxpy.DXGTable.make_column_desc("is_coding", "boolean"),
+            dxpy.DXGTable.make_column_desc("parent_id", "int64"),
+            dxpy.DXGTable.make_column_desc("frame", "int64"),
+            dxpy.DXGTable.make_column_desc("description", "string"),
+            dxpy.DXGTable.make_column_desc("chr", "string"),
+            dxpy.DXGTable.make_column_desc("lo", "int64"),
+            dxpy.DXGTable.make_column_desc("hi", "int64")
+        ])
+        genes_table.add_rows(data=[
+            ["transcript", 5, "mytranscript-noncoding", "+", False, -1, -1, "my test transcript", "chr1", 100, 200],
+            ["exon", 6, "", "+", False, 5, -1, "", "chr1", 100, 200],
+            ["gene", 54, "mygene-coding", "+", True, -1, -1, "my test gene", "chr1", 150, 200],
+            ["transcript", 55, "mytranscript-coding", "+", True, 54, -1, "my test transcript", "chr1", 150, 200],
+            ["CDS", 75, "", "+", True, 55, 0, "", "chr1", 150, 200]
+        ])
+        genes_table.set_details({
+            "original_contigset": {"$dnanexus_link": self.genome_id}
+        })
+        genes_table.close(block=True)
+
+        self.assertEquals(run('dx-genes-to-gtf {g}'.format(g=genes_table.get_id())),
+                          self.expected_gtf)
+
+
+class TestDXSamToMappings(DXTestCase):
+    def setUp(self):
+        super(TestDXSamToMappings, self).setUp()
+        self.tempdir = tempfile.mkdtemp()
+        self.expected_sam = """@SQ\tSN:chr1\tLN:249250621
+@RG\tID:0\tSM:Sample_0
+FOO.12345678\t0\t1\t54932369\t60\t7M1D93M\t*\t0\t0\tTAATAAGGTTGTTGTTGTTGTT\t1:1ADDDACFHA?HGFGIIE+<\tMD:Z:1A5^A93\tRG:Z:0
+"""
+        self.genome_id = makeGenomeObject()
+
+    def tearDown(self):
+        shutil.rmtree(self.tempdir)
+        super(TestDXSamToMappings, self).tearDown()
+
+    def test_mappings_to_sam_conversion(self):
+        mappings_table = dxpy.new_dxgtable([
+            dxpy.DXGTable.make_column_desc("sequence", "string"),
+            dxpy.DXGTable.make_column_desc("quality", "string"),
+            dxpy.DXGTable.make_column_desc("name", "string"),
+            dxpy.DXGTable.make_column_desc("status", "string"),
+            dxpy.DXGTable.make_column_desc("chr", "string"),
+            dxpy.DXGTable.make_column_desc("lo", "int32"),
+            dxpy.DXGTable.make_column_desc("hi", "int32"),
+            dxpy.DXGTable.make_column_desc("negative_strand", "boolean"),
+            dxpy.DXGTable.make_column_desc("error_probability", "uint8"),
+            dxpy.DXGTable.make_column_desc("qc_fail", "boolean"),
+            dxpy.DXGTable.make_column_desc("duplicate", "boolean"),
+            dxpy.DXGTable.make_column_desc("cigar", "string"),
+            dxpy.DXGTable.make_column_desc("template_id", "int64"),
+            dxpy.DXGTable.make_column_desc("read_group", "uint16"),
+            dxpy.DXGTable.make_column_desc("sam_field_MD", "string"),
+            dxpy.DXGTable.make_column_desc("sam_field_XN", "int32")
+        ])
+        mappings_table.add_rows(data=[[
+            "TAATAAGGTTGTTGTTGTTGTT",
+            "1:1ADDDACFHA?HGFGIIE+<",
+            "FOO.12345678",
+            "PRIMARY",
+            "1",
+            54932368,
+            54932390,
+            False,
+            60,
+            False,
+            False,
+            "7M1D93M",
+            289090731,
+            0,
+            "1A5^A93",
+            -2147483648
+        ]], part=1)
+        mappings_table.set_details({
+            "read_groups": [
+                {"num_singles": 1, "num_pairs": 0}
+            ],
+            "original_contigset": {"$dnanexus_link": self.genome_id}
+        })
+        mappings_table.close(block=True)
+
+        self.assertEquals(run('dx-mappings-to-sam {g}'.format(g=mappings_table.get_id())),
+                          self.expected_sam)
+
+
 if __name__ == '__main__':
     if 'DXTEST_FULL' not in os.environ:
         sys.stderr.write('WARNING: env var DXTEST_FULL is not set; tests that create apps will not be run\n')
